function mskedpg(pagename, dlgHandle, BlockHandle, position)
%MSKEDPG creates the three different pages of the Mask Editor.
%   MSKEDPG creates the three different pages (Icon, Parameter, and
%   Documentation) for the Mask Editor.

%   Copyright 1990-2002 The MathWorks, Inc.
%   $Revision: 1.30 $
%   Sanjai Singh  05-01-96

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Create the correct page %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch (pagename)

  case 'IconPage'
    IconPage(dlgHandle, BlockHandle, position)

  case 'ParamPage'
    ParamPage(dlgHandle, BlockHandle, position)

  case 'DocPage'
    DocPage(dlgHandle, BlockHandle, position)

  otherwise
    error(['Unknown page of the Mask Editor : ' pagename]);

end  % ... switch (pagename)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% End of main function : mskedpg %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  DocPage : Creates the Documentation Page of the Mask Editor  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  DocPage(dlgHandle, BlockHandle, position)
%
% DOCPAGE : Creates the Documentation Page of the Mask Editor
%           The Mask Type is always created by the first Page.
%           This page adds :
%           - Mask Description
%           - Mask Help
%
% Keep the handles of only the Edit Uicontrols because only they
% contain the needed information. Store in Figure's UserData.
%
  SyncData      = masksync('SyncDocPage', dlgHandle, BlockHandle);
  UIData        = masksync('UIGeom', dlgHandle, BlockHandle, position);
  Data          = get(dlgHandle, 'UserData');
  DocPageNumber = 'page3';

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % UIControls for the Description of the Masked Block. %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Style',                  'text', ...
      'Tag',                    DocPageNumber, ...
      'horizontalalignment',    'left',...
      'Position',               [15 (UIData.Height- 70) 125 20 ],...
      'Visible',                'off',...
      'String',                 'Block description:');

  Data.DocPage.MaskDescriptionHandle = uicontrol(...
      'Style',                  'edit', ...
      'Tag',                    DocPageNumber,  ...
      'Min',                    0,...
      'Max',                    2,...
      'HorizontalAlignment',    'left',...
      'Visible',                'off',...
      'String',                 {SyncData.DescriptionString},...
      'Position',               [20 (UIData.Height-160) (UIData.Width-15) 90], ...
      'backgroundColor',        [1 1 1]);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % UIControls for the Help of the Masked Block.        %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Tag',                    DocPageNumber,...
      'Style',                  'text',...
      'HorizontalAlignment',    'left',...
      'Position',               [15 (UIData.Height-205) 200  20],...
      'Visible',                'off',...
      'String',                 'Block help:');

  Data.DocPage.MaskHelpHandle = uicontrol(...
      'Tag',                    DocPageNumber,...
      'Style',                  'edit',...
      'Min',                    0,...
      'Max',                    2,...
      'horizontalAlignment',    'left',...
      'Position',               [20  (UIData.Height-315) (UIData.Width-15) 110],...
      'Visible',                'off',...
      'String',                 {SyncData.HelpString},...
      'BackgroundColor',        [1 1 1]);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Set the USERDATA of the Mask Editor %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  set(dlgHandle, 'UserData', Data);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% End of function : DocPage %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  ParamPage : Creates the Parameter Page of the Mask Editor    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  ParamPage(dlgHandle, BlockHandle, position)
%
% PARAMPAGE : Creates the Parameter Page of the Mask Editor
%             The Mask Type is always created by the first Page.
%             This page adds :
%             - Mask Initialization
%             - Mask Prompts
%             - Mask Param Types
%
% Keep the handles of only the Edit Uicontrols because only they
% contain the needed information. Store in Figure's UserData.
%
  SyncData        = masksync('SyncParamPage', dlgHandle, BlockHandle);
  UIData          = masksync('UIGeom', dlgHandle, BlockHandle, position);
  Data            = get(dlgHandle, 'UserData');
  ParamPageNumber = 'page2';

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Text UIControls for the Parameter ListBox %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Style',                     'text',...
      'Tag',                       ParamPageNumber, ...
      'Position',                  [90 (UIData.Height-49) 120 17 ],...
      'HorizontalAlignment',       'left',...
      'Visible',                   'off',...
      'String',                    'Prompt' );
  uicontrol(...
      'Style',                     'text',...
      'Tag',                       ParamPageNumber, ...
      'Position',                  [265 (UIData.Height-49) 55 17 ],...
      'HorizontalAlignment',       'left',...
      'Visible',                   'off',...
      'String',                    'Type' );
  uicontrol(...
      'Style',                     'text',...
      'Tag',                       ParamPageNumber, ...
      'Position',                  [320 (UIData.Height-49) 70 17 ],...
      'HorizontalAlignment',       'left',...
      'Visible',                   'off',...
      'String',                    'Variable' );

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% UIControl for Displaying the Variables in the ListBox %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Data.ParamPage.ParamListBox = uicontrol(...
      'style',                  'listbox',...
      'tag',                    ParamPageNumber,...
      'Callback',               'mskeddlg Listbox UpdateEdit',...
      'String',                 SyncData.ParamString,...
      'min',                    0,...
      'max',                    0,...
      'UserData',               SyncData.ParamData, ...
      'FontName',               UIData.ListboxFontname ,...
      'FontSize',               UIData.ListboxFontsize,...
      'FontWeight',             'normal',...
      'Visible',                'off',...
      'Position',               [90 (UIData.Height-160) 310 110]);


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% UIControls for Adding, Deleting and Moving Parameters in the ListBox %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Data.ParamPage.NewButtonHandle = uicontrol(...
      'Style',                 'pushbutton',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [15 (UIData.Height - 71) 70 20 ],...
      'HorizontalAlignment',   'center',...
      'Visible',               'off',...
      'Callback',              'mskeddlg Listbox New',...
      'String',                'Add' );

  Data.ParamPage.DeleteButtonHandle = uicontrol(...
      'Style',                 'pushbutton',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [15 (UIData.Height - 99) 70 20 ],...
      'HorizontalAlignment',   'center',...
      'Visible',               'off',...
      'enable',                SyncData.EnableDeleteButton,...
      'Callback',              'mskeddlg Listbox Delete',...
      'String',                'Delete' );

  Data.ParamPage.UpButtonHandle = uicontrol(...
      'Style',                 'pushbutton',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [15 (UIData.Height - 127) 70 20 ],...
      'HorizontalAlignment',   'center',...
      'Visible',               'off',...
      'enable',                SyncData.EnableUpButton,...
      'Callback',              'mskeddlg Listbox Up',...
      'String',                'Up');

  Data.ParamPage.DownButtonHandle = uicontrol(...
      'Style',                 'pushbutton',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [15 (UIData.Height - 155) 70 20 ],...
      'HorizontalAlignment',   'center',...
      'Visible',               'off',...
      'enable',                SyncData.EnableDownButton,...
      'Callback',              'mskeddlg Listbox Down',...
      'String',                'Down' );


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Text and Edit UIControls for the Prompt %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Data.ParamPage.MaskPStringHandle = uicontrol(...
      'Style',                 'text',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [15 (UIData.Height - 182) 50 15],...
      'HorizontalAlignment',   'left',...
      'Visible',               'off',...
      'enable',                SyncData.EnablePromptText,...
      'String',                'Prompt: ' );

  Data.ParamPage.MaskPromptsHandle = uicontrol(...
      'Style',                 'edit',...
      'Tag',                   ParamPageNumber, ...
      'min',                   0,...
      'max',                   1,...
      'Callback',              'mskeddlg Listbox Edit',...
      'String',                SyncData.PromptString, ...
      'Position',              [65  (UIData.Height - 183) 130 UIData.EditSize],...
      'Visible',               'off',...
      'enable',                SyncData.EnablePromptEdit,...
      'HorizontalAlignment',   'left',...
      'backgroundColor',       [1 1 1] );

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Text and Edit UIControls for the Variable %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Data.ParamPage.MaskVStringHandle = uicontrol(...
      'Style',                 'text',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [15 (UIData.Height - 212) 50 20 ],...
      'HorizontalAlignment',   'left',...
      'Visible',               'off',...
      'enable',                SyncData.EnableVariableText,...
      'String',                'Variable: ' );

  Data.ParamPage.MaskVariablesHandle = uicontrol(...
      'Style',                 'edit',...
      'Tag',                   ParamPageNumber, ...
      'min',                   0,...
      'max',                   1,...
      'Callback',              'mskeddlg Listbox Edit',...
      'String',                SyncData.FirstVariable,...
      'Visible',               'off',...
      'enable',                SyncData.EnableVariableEdit,...
      'Position',              [65 (UIData.Height - 208) 130 UIData.EditSize ],...
      'HorizontalAlignment',   'left',...
      'backgroundColor',       [1 1 1] );

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Text and Edit UIControls for the Parameter Type %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Data.ParamPage.MaskParamStringHandle = uicontrol(...
      'Style',                 'text',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [230 (UIData.Height - 187) 80 20 ],...
      'horizontalalignment',   'left',...
      'enable',                SyncData.EnableControlText,...
      'Visible',               'off',...
      'String',                'Control type:' );

  Data.ParamPage.MaskParamTypeHandle = uicontrol(...
      'Style',                 'popup',...
      'String',                'Edit|Checkbox|Popup',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [310 UIData.TypePopupOffset 90 UIData.PopupSize],...
      'Visible',               'off',...
      'enable',                SyncData.EnableControlPopup,...
      'Backgroundcolor',       UIData.PopupColor,...
      'value',                 SyncData.Value,...
      'Callback',              'mskeddlg Listbox Edit'...
      );

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Text and Edit UIControls for the Evaluation Type %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Data.ParamPage.MaskEvalStringHandle = uicontrol(...
      'Style',                 'text',...
      'Tag',                   ParamPageNumber, ...
      'Position',              [230 (UIData.Height - 212) 80 20 ],...
      'HorizontalAlignment',   'left',...
      'Visible',               'off',...
      'enable',                SyncData.EnableAssgnText,...
      'String',                'Assignment:' );

  Data.ParamPage.MaskEvalHandle = uicontrol(...
      'Style',                 'popup',...
      'Tag',                   ParamPageNumber, ...
      'Visible',               'off',...
      'enable',                SyncData.EnableAssgnPopup,...
      'String',                'Evaluate|Literal',...
      'Backgroundcolor',       UIData.PopupColor,...
      'Value',                 SyncData.FirstEvalType,...
      'Callback',              'mskeddlg Listbox Edit',...
      'Position',              [310 UIData.EvalPopupOffset 90 UIData.PopupSize]...
      );

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Text and Edit UIControls for the Popup Strings   %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Data.ParamPage.MaskPopupText = uicontrol(...
      'Style',                'text',...
      'Tag',                  ParamPageNumber, ...
      'enable',               SyncData.EnablePopupString,...
      'Position',             [15 (UIData.Height - 234) 90 15 ],...
      'HorizontalAlignment',  'left',...
      'Visible',              'off',...
      'String',               'Popup strings:' );

  Data.ParamPage.MaskPopupStrings = uicontrol(...
      'Style',                'edit',...
      'Tag',                  ParamPageNumber, ...
      'Position',             [110 (UIData.Height - 234) (UIData.Width-105) UIData.EditSize],...
      'String',               SyncData.FirstPopupString,...
      'enable',               SyncData.EnablePopupString,...
      'Visible',              'off',...
      'HorizontalAlignment',  'left',...
      'Callback',             'mskeddlg Listbox Edit',...
      'backgroundColor',      [1 1 1]...
      );

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Text and Edit UIControls for the Initialization Commands %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Tag',                  ParamPageNumber,...
      'Style',                'text',...
      'HorizontalAlignment',  'left',...
      'Position',             [15 (UIData.Height-260) 200  15],...
      'Visible',              'off',...
      'String',               'Initialization commands:');

  Data.ParamPage.MaskInitCommandsHandle = uicontrol(...
      'Tag',                  ParamPageNumber,...
      'Style',                'edit',...
      'HorizontalAlignment',  'left',...
      'Min',                  0,...
      'Max',                  2,...
      'Position',             [20 (UIData.Height-317) (UIData.Width-15) 55],...
      'Visible',              'off',...
      'String',               SyncData.InitString,...
      'BackgroundColor',      [1 1 1]);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Set the USERDATA of the Mask Editor %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  set(dlgHandle, 'UserData', Data);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% End of function : ParamPage %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  IconPage : Creates the Icon Page of the Mask Editor          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  IconPage(dlgHandle, BlockHandle, position)
%
% ICONPAGE : Creates the Icon Page of the Mask Editor
%           The Mask Type is always created by the first Page.
%           This page adds :
%           - Mask Display
%           - Mask Rotate Icon
%           - Mask Frame Icon
%           - Mask Opaque Icon
%           - Mask Drawing Coords
%
% Keep the handles of only the Edit Uicontrols because only they
% contain the needed information. Store in Figure's UserData.
%
  SyncData        = masksync('SyncIconPage',dlgHandle, BlockHandle);
  UIData          = masksync('UIGeom', dlgHandle, BlockHandle, position);
  Data            = get(dlgHandle, 'UserData');
  IconPage        = 'page1';
  IconPageVisible = 'on';

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Since the following uicontrols appears on all 3 screens, it      %%
  %% should be placed on the screen that appears first. In this case, %%
  %% I've changed it from the Doc Page to the Icon Page.              %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % The UIControls for the Block Type
  uicontrol(...
      'Style',                   'text', ...
      'Position',                [15 (UIData.Height-28) 75 20 ],...
      'HorizontalAlignment',     'left',...
      'String',                  'Mask type: ' );

  Data.IconPage.MaskTypeHandle = uicontrol(...
      'Style',                   'edit', ...
      'Tag',                     'page0',  ...
      'HorizontalAlignment',     'left',...
      'String',                  SyncData.TypeString,...
      'Position',                [90 (UIData.Height - 23) (UIData.Width-85) UIData.EditSize],...
      'backgroundColor',         [1 1 1]);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%% END OF UNIVERSAL UICONTROLS %%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %    UIControls for Drawing Commands        %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Tag',                      IconPage,...
      'Style',                    'text',...
      'HorizontalAlignment',      'left',...
      'Position',                 [15 (UIData.Height-57) 160  20],...
      'Visible',                  IconPageVisible,...
      'String',                   'Drawing commands:');

  Data.IconPage.MaskDisplayHandle = uicontrol(...
      'Tag',                      IconPage,...
      'Style',                    'edit',...
      'HorizontalAlignment',      'left',...
      'Min',                      0,...
      'Max',                      2,...
      'Position',                 [20 (UIData.Height-195) (UIData.Width-15) 138],...
      'Visible',                  IconPageVisible,...
      'String',                   {SyncData.DisplayString},...
      'BackgroundColor',          [1 1 1]);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %    UIControl for the FrameIcon Popup   %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Tag',                      IconPage,...
      'Style',                    'text',...
      'horizontalalignment',      'right',...
      'Visible',                  IconPageVisible,...
      'Position',                 [145 (UIData.Height-232) 140 UIData.RadioHeight],...
      'String',                   'Icon frame:');

  Data.IconPage.MaskFrameIconHandle = uicontrol(...
      'Tag',                      IconPage,...
      'Style',                    'popup',...
      'Position',                 [290 UIData.FrameOffset 110 UIData.PopupSize],...
      'Value',                    SyncData.FrameValue,...
      'Backgroundcolor',          UIData.PopupColor,...
      'HorizontalAlignment',      'left',...
      'Visible',                  IconPageVisible,...
      'String',                   'Invisible|Visible');

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %    UIControl for the OpaqueIcon Popup  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Tag',                   IconPage,...
      'Style',                 'text',...
      'horizontalalignment',   'right',...
      'Visible',               IconPageVisible,...
      'Position',              [145 (UIData.Height-262) 140 UIData.RadioHeight],...
      'String',                'Icon transparency:');


  Data.IconPage.MaskOpaqueIconHandle = uicontrol(...
      'Tag',                   IconPage,...
      'Style',                 'popup',...
      'Position',              [290 UIData.OpaqueOffset 110 UIData.PopupSize],...
      'Value',                 SyncData.OpaqueValue,...
      'Backgroundcolor',       UIData.PopupColor,...
      'HorizontalAlignment',   'left',...
      'Visible',               IconPageVisible,...
      'String',                'Transparent|Opaque');

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %    UIControl for the RotateIcon Popup  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  uicontrol(...
      'Tag',                   IconPage,...
      'Style',                 'text',...
      'horizontalalignment',   'right',...
      'Position',              [145 (UIData.Height-292) 140 UIData.RadioHeight],...
      'Visible',               IconPageVisible,...
      'String',                'Icon rotation:'  );

  Data.IconPage.MaskRotateIconHandle = uicontrol(...
      'Tag',                   IconPage,...
      'Style',                 'popup',...
      'Position',              [290 UIData.RotateOffset 110 UIData.PopupSize],...
      'Value',                 SyncData.RotateValue,...
      'Backgroundcolor',       UIData.PopupColor,...
      'HorizontalAlignment',   'left',...
      'Visible',               IconPageVisible,...
      'String',                'Fixed|Rotates');

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %    UIControl for the DrawingCoords Popup  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  iconCoordsText = uicontrol(...
      'Tag',                   IconPage,...
      'Style',                 'text',...
      'horizontalalignment',   'right',...
      'Position',              [135 (UIData.Height-322) 150 UIData.RadioHeight],...
      'Visible',               IconPageVisible,...
      'String',                'Drawing coordinates:');

  Data.IconPage.MaskDrawingCoordsHandle = uicontrol(...
      'Tag',                   IconPage,...
      'Style',                 'popup',...
      'Position',              [290 UIData.CoordsOffset 110 UIData.PopupSize],...
      'Backgroundcolor',       UIData.PopupColor,...
      'Value',                 SyncData.NormalValue,...
      'Visible',               IconPageVisible,...
      'HorizontalAlignment',   'left',...
      'String',                'Pixel|Autoscale|Normalized');

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% Set the USERDATA of the Mask Editor %%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  set(dlgHandle, 'UserData', Data);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% End of function : IconPage %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
