function handleStruct = sigbuilder_beveled_frame(fig,pos,style,frameH)
%SIGBUILDER_BEVELED_FRAME - Create a 3 dimensional frame by using
% different light and dark shades.
%
%

%  Copyright 1984-2002 The MathWorks, Inc.
%  $Revision: 1.2 $  $Date: 2002/04/10 18:29:23 $
    
    if nargin<3
        style = 'NarrowTrans';
    end
    
    if nargin<4
        frameH = [];
    end
    
    switch(style)
    case 'NarrowTrans'
        isWide = 0;
        isOpaque = 0;
    case 'NarrowOpaq'
        isWide = 0;
        isOpaque = 1;
    case 'WideTrans'
        isWide = 1;
        isOpaque = 0;
    case 'WideOpaq'
        isWide = 1;
        isOpaque = 1;
    end
    
    
    lineWidth = 1;
    lineHeight = 1;
    
    DefaultUIColor	=	[0.502, 0.502, 0.502]; 
    FocusColor		=	[0.502, 0.502, 0.502];
    NormalColor		=	[0.0, 0.0, 0.0];
    AntiqueWhite	=	[1.0, 0.9608, 0.8842];
    DropColor		=	[0.0, 0.0, 0.0];
    DragColor		=	[0.502, 0.502, 0.502];
    IcedColor		=	[0.725, 0.725, 0.725];
    White			=	[1.0, 1.0, 1.0];
    Black			=	[0.0, 0.0, 0.0];
    Light			=	[0.725, 0.725, 0.725];
    LightGrey		=	[0.9, 0.9, 0.9];
    Dark			=	[0.502, 0.502, 0.502];
    

    % If the units are points we need to find
    % a conversion
    figUnits = get(fig,'Units');
    switch(figUnits)
    case 'points'

        screenUnits = get(0,'Units');
        set(0,'Units','points');
        pointSize = get(0,'ScreenSize');
        set(0,'Units','pixels');
        pixelsSize = get(0,'ScreenSize');
        set(0,'Units',screenUnits);
        
        pixels2points = pointSize./pixelsSize;
        pixels2points = mean(pixels2points(3:4));

        lineWidth = lineWidth*pixels2points;
        lineHeight = lineHeight*pixels2points;
    case 'pixels'
        % Do nothing
    otherwise
        error('Units must be pixels or points');
    end


    % Determine positions
    leftOuter = [ 	pos(1), ...
                	pos(2), ...
                	lineWidth, ...
                	pos(4)];
    
    topOuter = [ 	pos(1), ...
                	pos(2)+pos(4)-lineHeight, ...
                	pos(3), ...
                	lineHeight];
    
    bottomOuter = [ pos(1)+lineWidth, ...
                	pos(2), ...
                	pos(3)-lineWidth, ...
                	lineHeight];

    rightOuter = [  pos(1)+pos(3)-lineWidth, ...
                	pos(2), ...
                	lineWidth, ...
                	pos(4)-lineHeight];


    leftInner = [ 	pos(1)+lineWidth, ...
                	pos(2)+lineHeight, ...
                	lineWidth, ...
                	pos(4)-2*lineHeight];
    
    topInner = [ 	pos(1)+lineWidth, ...
                	pos(2)+pos(4)-2*lineHeight, ...
                	pos(3)-2*lineWidth, ...
                	lineHeight];
    
    bottomInner = [ pos(1)+2*lineWidth, ...
                	pos(2)+lineHeight, ...
                	pos(3)-3*lineWidth, ...
                	lineHeight];

    rightInner = [  pos(1)+pos(3)-2*lineWidth, ...
                	pos(2)+lineHeight, ...
                	lineWidth, ...
                	pos(4)-3*lineHeight];

    if isWide
        framePos = pos + [2 0 -4 0]*lineWidth + [0 2 0 -4]*lineHeight;
        outerDark = Black;
        outerLight = Light;
    else
        framePos = pos + [1 0 -2 0]*lineWidth + [0 1 0 -2]*lineHeight;
        outerDark = Dark;
        outerLight = White;
    end


    if isOpaque
        if ~isempty(frameH)
            handleStruct.frame = frameH;
            set(frameH, ... 
                                'BackgroundColor',  LightGrey, ...
                                'ForegroundColor',  LightGrey, ...
                                'Units',         figUnits, ...
                                'Position',         framePos, ...
                                'Style',            'frame');
        else
            handleStruct.frame = uicontrol( 'Parent',           fig, ...
                                'BackgroundColor',  LightGrey, ...
                                'ForegroundColor',  LightGrey, ...
                                'Units',         figUnits, ...
                                'Position',         framePos, ...
                                'Style',            'frame');
        end

    end
    % Create the dark lines
    handleStruct.left = uicontrol( 'Parent',           fig, ...
                            'BackgroundColor',  outerDark, ...
                            'Units',         figUnits, ...
                            'Position',         leftOuter, ...
                            'Style',            'text');

    handleStruct.top = uicontrol(  'Parent',           fig, ...
                            'BackgroundColor',  outerDark, ...
                            'Units',         figUnits, ...
                            'Position',         topOuter, ...
                            'Style',            'text');

    % Create the white lines
    handleStruct.right = uicontrol( 'Parent',           fig, ...
                            'BackgroundColor',  outerLight, ...
                            'Units',         figUnits, ...
                            'Position',         rightOuter, ...
                            'Style',            'text');

    handleStruct.bottom = uicontrol(   'Parent',           fig, ...
                                'BackgroundColor',  outerLight, ...
                                'Units',         figUnits, ...
                                'Position',         bottomOuter, ...
                                'Style',            'text');


    if isWide
        % Create the dark lines
        handleStruct.leftInner = uicontrol( 'Parent',           fig, ...
                                'BackgroundColor',  Dark, ...
                                'Units',         figUnits, ...
                                'Position',         leftInner, ...
                                'Style',            'text');
    
        handleStruct.topInner = uicontrol(  'Parent',           fig, ...
                                'BackgroundColor',  Dark, ...
                                'Units',         figUnits, ...
                                'Position',         topInner, ...
                                'Style',            'text');
    
        % Create the white lines
        handleStruct.rightInner = uicontrol( 'Parent',           fig, ...
                                'BackgroundColor',  White, ...
                                'Units',         figUnits, ...
                                'Position',         rightInner, ...
                                'Style',            'text');
    
        handleStruct.bottomInner = uicontrol(   'Parent',           fig, ...
                                    'BackgroundColor',  White, ...
                                    'Units',         figUnits, ...
                                    'Position',         bottomInner, ...
                                    'Style',            'text');
    end