function diag=sllastdiagnostic(newDiag)
%SLLASTDIAGNOSTIC Simulink last diagnostic message.
%   SLLASTDIAGNOSTIC by itself returns a Simulink diagnostic structure array
%   containing the last diagnostic(s) generated by Simulink.  The fields of
%   the diagnostic structure are:
%
%     Type        'error' or 'warning'
%     MessageID   the message ID for the diagnostic (e.g., 'SL_InvSimulinkObjectName')
%     Message     the diagnostic message
%     Handle      Simulink object handle(s) associated with the diagnostic
%
%   SLLASTDIAGNOSTIC([]) resets the Simulink last diagnostic so that it will return
%   an empty array until the next Simulink diagnostic is encountered.
%
%   SLLASTDIAGNOSTIC(DIAGSTRUCT) will set the Simulink last diagnostic(s) to those
%   specified in the DIAGSTRUCT.
%   
%   See also SLLASTERROR, SLLASTWARNING.

%   Copyright 1990-2002 The MathWorks, Inc.
%   $Revision: 1.5 $

%
% if no input args, return the last diagnostic, otherwise, set it
%
if nargin == 0,
  diag = get_param(0,'LastDiagnostic');
else
  %
  % validate that the Type field is either 'error' or 'warning'
  %
  if ~isempty(newDiag),
    type = { newDiag(:).Type };
    type(find(strcmp(type, 'error'))) = [];
    type(find(strcmp(type, 'warning'))) = [];
    if ~isempty(type),
      error(['Type field of the Simulink diagnostic structure array must be ' ...
             '''error'' or ''warning''.']);
    end
  end
  
  set_param(0,'LastDiagnostic', newDiag);
end
