function sltipalert(varargin)
% SLTIPALERT Manages and displays the Simulink tip alert window
%
%   SLTIPALERT('CREATE', DIALOGDESC, DIALOGTITLE, FEATURETIPPRESKEY)
%    Creates the TipAlert dialog:
%      DIALOGDESC is the description of the feature tip
%      DIALOGTITLE is the dialog title
%      FEATURETIPPRESKEY is the preferencs key setting
%
%   SLTIPALERT('SHOW', FEATURETIPPRESKEY)  sets feature tip to on.
%   SLTIPALERT('NEVER', FEATURETIPPRESKEY) sets the feature tip to off.


%   Copyright 2001-2002 The MathWorks, Inc.
%   $Revision: 1.4.2.1 $
%   Ricardo Monteiro  03/01/2001

error(nargchk(2, 4, nargin))

if ~usejava('mwt')
    error('Java is not currently enabled on this platform');
end

Action = lower(varargin{1});

if(nargin == 4)
  strDlgDesc  = varargin{2};  
  strTitle    = varargin{3};
  strPrefsKey = varargin{4};
elseif (nargin == 2)
  strPrefsKey = varargin{2};
elseif (nargin == 3)
 error('Invalid input arguments')
else
  return
end

switch (Action)
 
 case 'create'
  error(nargchk(4, 4, nargin));
  % Launch the TipAlert dialog according to the prefeces file settings
  prefsKeyStatus =  com.mathworks.services.Prefs.getBooleanPref(strPrefsKey);
  if(prefsKeyStatus)
     i_slTipAlertCreate(strDlgDesc, strTitle, strPrefsKey);
  else
    return
  end
  
 case 'never'
  error(nargchk(2, 2, nargin));
  com.mathworks.services.Prefs.setBooleanPref(strPrefsKey,0);
 
 case 'show'
  error(nargchk(2, 2, nargin));
  com.mathworks.services.Prefs.setBooleanPref(strPrefsKey,1);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  i_slTipAlertCreate( strDlgDesc, strTitle, strPrefsKey)

FigPos=get(0,'DefaultFigurePosition');

if ispc
  FigWidth=250;
  dontShowAgainCheckBoxPos = [10 40 160 11.25];
  ButtonOnePos = [139 7 48 16];
  ButtonTwoPos = [194 7 48 16];
else
  FigWidth=300;
  dontShowAgainCheckBoxPos = [10 30 180 20];
  ButtonOnePos = [190 7 48 16];
  ButtonTwoPos = [242 7 48 16];
end

FigHeight=110;
FigPos(3:4)=[FigWidth FigHeight];
AlertFig=dialog(                                               ...
               'Visible'         ,'off'                       ,...
               'Name'            , strTitle                   ,...
               'Pointer'         ,'arrow'                     ,...
               'Units'           ,'points'                    ,...
               'Position'        , FigPos                     ,...
               'UserData'        , strPrefsKey                ,...
               'IntegerHandle'   ,'off'                       ,...
               'Resize'          ,'on'                        ,... 
	       'WindowStyle'     ,'normal'                    ,... 
               'HandleVisibility','callback'                  ,...
	       'DeleteFcn',       {@i_slTipAlertSetPresfs}    ,...	      
	       'Tag'             , 'FigureTitle'               ...
               );

BtnFontSize=get(0,'FactoryUIControlFontSize');
BtnFontName=get(0,'FactoryUIControlFontName');
helpButtonString='Help';
ButtonTag='Help';


BtnHandle(1)=uicontrol(AlertFig               , ...
		       'Style'                ,'pushbutton'        ,...
		       'Units'                ,'points'            ,...
		       'Position'             ,ButtonOnePos        ,...
		       'CallBack'             ,{@i_slTipAlertHelp} ,...
		       'String'               , helpButtonString   ,...
		       'HorizontalAlignment'  ,'center'            ,...
		       'FontUnits'            ,'points'            ,...
		       'FontSize'             ,BtnFontSize         ,...
		       'FontName'             ,BtnFontName         ,...
		       'Tag'                  ,ButtonTag           ...
		       );

CloseButtonString='Close';
ButtonTag='Close';
BtnHandle(2) = uicontrol(AlertFig                                                ,...
			 'Style'              ,'pushbutton'                      ,...
			 'Units'              ,'points'                          ,...
			 'Position'           , ButtonTwoPos                     ,...
			 'CallBack'           ,'delete(get(0,''CurrentFigure''))',...
			 'String'             ,CloseButtonString                 ,...
			 'HorizontalAlignment','center'                          ,...
			 'FontUnits'          ,'points'                         ,...
			 'FontSize'           ,BtnFontSize                       ,...
			 'FontName'           ,BtnFontName                       ,...
			 'Tag'                ,ButtonTag                         ...
			 );
 
msgPosition = [36 53 183 51];			 
MsgHandle=uicontrol(AlertFig            , ...
		    'Style'              ,'text'         , ...
		    'Units'              ,'points'       , ...
		    'Position'           , msgPosition   , ... 
		    'String'             ,{' '}          , ...
		    'Tag'                ,'strDlgDesc'   , ...
		    'HorizontalAlignment','left'         , ...    
		    'FontUnits'          ,'points'       , ...
		    'FontWeight'         ,'bold'         , ...
		    'FontSize'           ,BtnFontSize    , ...
		    'FontName'           ,BtnFontName     ...
		    );

if ~iscell(strDlgDesc),strDlgDesc=cellstr(strDlgDesc);end
[WrapString,NewMsgTxtPos]=textwrap(MsgHandle,strDlgDesc,65);

delete(MsgHandle);
AxesHandle=axes('Parent',AlertFig,'Position',[0 0 1 1],'Visible','off');

MsgHandle=text( ...
    'Parent'              ,AxesHandle         , ...
    'Units'               ,'points'           , ...
    'FontUnits'           ,'points'           , ...
    'FontSize'            ,BtnFontSize        , ...
    'FontName'            ,BtnFontName        , ...
    'HorizontalAlignment' ,'left'             , ...
    'VerticalAlignment'   ,'bottom'           , ...
    'HandleVisibility'    ,'callback'         , ...
    'Position'            ,[10 70 0]          , ...
    'String'              ,WrapString         , ...
    'Tag'                 ,'strDlgDesc'       ...
    );

dontShowAgainStr = 'Do not show this message again';
dontShowAgainCheckBox = uicontrol(AlertFig            , ...
				  'Style'              ,'checkbox'               , ...
				  'Units'              ,'points'                 , ...
				  'Position'           , dontShowAgainCheckBoxPos, ... 
				  'String'             , dontShowAgainStr        ,...  
				  'Tag'                ,'DontShowAgain'          , ...
				  'HorizontalAlignment','left'         , ...    
				  'FontUnits'          ,'points'       , ...
				  'FontWeight'         ,'bold'         , ...
				  'FontSize'           ,BtnFontSize    , ...
				  'FontName'           ,BtnFontName     ...
				  );

ScreenUnits=get(0,'Units');
set(0,'Units','points');
ScreenSize=get(0,'ScreenSize');
set(0,'Units',ScreenUnits);

FigPos(1)=(ScreenSize(3)-FigWidth)/2;
FigPos(2)=(ScreenSize(4)-FigHeight)/2;

set(AlertFig ,'Position',FigPos);
set(AlertFig ,'Visible','on');
drawnow;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  i_slTipAlertSetPresfs(HandleAlertFig,ed)

handles = guihandles(HandleAlertFig);

  strPrefsKey = get(handles.FigureTitle,'UserData');
  if (get(handles.DontShowAgain,'Value'))
    com.mathworks.services.Prefs.setBooleanPref(strPrefsKey,0)
  else
    com.mathworks.services.Prefs.setBooleanPref(strPrefsKey,1)
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ad = i_slTipAlertHelp(obj, evd)

helpview(fullfile(docroot,'mapfiles','simulink.map'), 'autoconnect')
