function [vectHGHandle, figHeight] = dlg_common_uis( fig, figWidth, c, objectId, yScale, editLabel, editTag)
%[VECTHGHANDLE, FIGHEIGHT] = DLG_COMMON_UIS( FIG, FIGWIDTH, CONSTANTS, OBJECTID, YSCALE, EDITLABEL, EDITTAG )

%   E. Mehran Mestchian
%   Copyright 1995-2002 The MathWorks, Inc.
%   $Revision: 1.16.2.1 $  $Date: 2004/04/15 00:56:48 $

   if nargin<5
      yScale = 1;
   end

   if nargin<6
      editLabel = 'Description:';
      editTag = 'DESCRIPTION';
   end

	% ID Display Field
	idString = ['ID# ' sprintf('%d',objectId)];

%   offset = [c.xmargin c.ymargin+0.5*(c.textHeight-c.fontSize)];
   offset = [c.xmargin c.ymargin];
   position = [dlg_origin(fig,'BL',offset), 6*c.fontSize c.textHeight];
   figHeight = position(2)+position(4);

	vectHGHandle = uicontrol('Style','text'...
      ,'Parent',fig...
      ,'Foreground','blue'...
      ,'Enable','off'...
      ,'Background',c.backColor...
      ,'String',idString...
      ,'UserData',dlg_data(fig,'BL',offset,0,0,'gotoStateflow')...
      ,'Position',position...
   );

	%%%%%%%%%%%%%%%%%%%%%%%%
	% Apply Button

   offset = [-(c.buttonW+c.xmargin),c.ymargin];
   position = [dlg_origin(fig,'BL',[(figWidth-c.buttonW-c.xmargin),c.ymargin]) c.buttonW c.buttonH];
   figHeight = max(figHeight,position(2)+position(4));

   vectHGHandle(end+1) = uicontrol('Style','push'...
      ,'Parent',fig...
      ,'String','Apply'...
      ,'Enable','on'...
      ,'UserData',dlg_data(fig, 'BR', offset, 0,0, 'apply') ...
      ,'Position',position...
      ,'Background',c.backColor...
      ,'Foreground','black'...
      ,'Tag','APPLYBUTTON'...
      ,'HorizontalAlign','center'...
	);

	%%%%%%%%%%%%%%%%%%%%%%%%
	% Help Button

   offset = [-(c.buttonW+c.buff),0];
   position = [dlg_origin(vectHGHandle(end),'BL',offset) c.buttonW c.buttonH];
   figHeight = max(figHeight,position(2)+position(4));

   vectHGHandle(end+1) = uicontrol('Style','push'...
      ,'Parent',fig...
      ,'String','Help'...
      ,'UserData',dlg_data(vectHGHandle(end), 'BL', offset, 0,0) ...
      ,'Position',position...
      ,'Background',c.backColor...
      ,'Foreground','black'...
      ,'Enable','on'...
		,'Callback','sfhelp'...
      ,'Tag','HELPBUTTON'...
      ,'HorizontalAlign','center'...
	);

	%%%%%%%%%%%%%%%%%%%%%%%%
	% Cancel Button

   offset = [-(c.buttonW+c.buff),0];
   position = [dlg_origin(vectHGHandle(end),'BL',offset) c.buttonW c.buttonH];
   figHeight = max(figHeight,position(2)+position(4));

   vectHGHandle(end+1) = uicontrol('Style','push'...
      ,'Parent',fig...
      ,'String','Cancel'...
      ,'UserData',dlg_data(vectHGHandle(end), 'BL', offset, 0,0, 'cancel') ...
      ,'Position',position...
      ,'Background',c.backColor...
      ,'Foreground','black'...
      ,'Enable','on'...
      ,'Tag','CANCELBUTTON'...
      ,'HorizontalAlign','center'...
	);

	%%%%%%%%%%%%%%%%%%%%%%%%
	% OK Button

   offset = [-(c.buttonW+c.buff),0];
   position = [dlg_origin(vectHGHandle(end),'BL',offset) c.buttonW c.buttonH];
   figHeight = max(figHeight,position(2)+position(4));

   vectHGHandle(end+1) = uicontrol('Style','push'...
      ,'Parent',fig...
      ,'String', 'OK'...
      ,'Enable','on'...
      ,'UserData',dlg_data(vectHGHandle(end), 'BL', offset, 0,0, 'OK') ...
      ,'Position',position...
      ,'Background',c.backColor...
      ,'Foreground','black'...
      ,'Tag','OKBUTTON'...
      ,'HorizontalAlign','center'...
	);

	if ~strcmp(editLabel,'none')

		%%%%%%%%%%%%%%%%%%%%%%%%
		% Document Label
	   offset = [0,c.buff];
	   position = [dlg_origin(vectHGHandle(1),'TL',offset) dlg_str_width('Document Link:',fig) c.textHeight];
	   figHeight = max(figHeight,position(2)+position(4));
		vectHGHandle(end+1) = uicontrol('Style','text'...
		   ,'Parent',fig...
		   ,'Foreground','blue'...
	      ,'Enable','inactive'...
		   ,'Background',c.backColor...
		   ,'String','Document Link:'...
			,'FontWeight','normal'...
		   ,'UserData',dlg_data(vectHGHandle(1),'TL',offset,0,0,'gotoDocument')...
		   ,'Position',position...
	   );
	
		%%%%%%%%%%%%%%%%%%%%%%%%
		% Document Edit Field
	
	   offset = [c.buff,0];
	   position = [dlg_origin(vectHGHandle(end),'BR',offset) 0 c.popupH];
	   figHeight = max(figHeight,position(2)+position(4));
	   position(3) = figWidth - position(1) - c.xmargin;
	
	   vectHGHandle(end+1) = uicontrol('Style','edit'...
	      ,'Parent',fig ...
	      ,'Enable','on' ...
	      ,'UserData',dlg_data(vectHGHandle(end), 'BR', offset, 1,0)...
	      ,'Position',position ...
	      ,'Background','white'...
	      ,'Max',1 ...
	      ,'Tag','DOCUMENT'...
	   );
	
		%%%%%%%%%%%%%%%%%%%%%%%%
		% Edit Field
	
	   offset = [0, c.popupH+c.buff];
	   position = [dlg_origin(vectHGHandle(end-1),'BL',offset) figWidth-2*c.xmargin 3*c.fontSize];
	   figHeight = max(figHeight,position(2)+position(4));
	   descriptionWidth = position(3);
	
	   vectHGHandle(end+1) = uicontrol('Style','edit'...
	      ,'Parent',fig ...
	      ,'Enable','on' ...
	      ,'UserData',dlg_data(vectHGHandle(end-1), 'BL', offset, 1,yScale)...
	      ,'Position',position ...
	      ,'Background','white'...
	      ,'Max',1000 ...
	      ,'Tag',editTag...
	   );
	
		%%%%%%%%%%%%%%%%%%%%%%%%
		% Edit Label
	   offset = [0, c.reallySmallBuff];
	   position = [dlg_origin(vectHGHandle(end),'TL',offset) descriptionWidth c.textHeight];
	   figHeight = max(figHeight,position(2)+position(4));
	
			vectHGHandle(end+1) = uicontrol('Style','text'...
	      ,'Parent',fig...
	      ,'Foreground','black'...
	      ,'Background',c.backColor...
	      ,'String',editLabel...
			,'FontWeight','bold'...
	      ,'UserData',dlg_data(vectHGHandle(end),'TL',offset,0,0)...
	      ,'Position',position...
	      );
	end

